import {
  ClassType,
} from '@overflow/commons/core/type';

import {
  Class,
} from '@overflow/commons/core/reflect';

import {
  InstanceDefinition,
  InstanceFactory,
} from '@overflow/commons/di/factory';

export class ApplicationContext {
  private _instanceFactory: InstanceFactory;

  public constructor(injectables: ClassType[]) {
    this._instanceFactory = new InstanceFactory();
    this.initializeInjectables(injectables);
  }

  private initializeInjectables(injectables: ClassType[]): void {
    if (undefined === injectables) {
      return;
    }
    injectables.forEach(injectable => {
      let name = injectable.name;
      let definition: InstanceDefinition = new InstanceDefinition(injectable);
      this._instanceFactory.registerInstanceDefinition(name, definition);
    });
  }

  /**
   * instanceFactory
   */
  public get instanceFactory(): InstanceFactory {
    return this._instanceFactory;
  }


}
