import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  InstanceNameType,
} from '@overflow/commons/di/type';

export interface InjectAnnotationAttributes {
  name?: InstanceNameType;
}

export class InjectAnnotation extends Annotation {
  public readonly attributes: InjectAnnotationAttributes = {

  };

  public constructor(name: InstanceNameType | InjectAnnotationAttributes) {
    super();

    if (undefined === name) {
      return;
    }

    if (TypeUtil.isString(name)) {
      this.attributes.name = <InstanceNameType>name;
    } else {
      this.copyAttributes(this.attributes, <InjectAnnotationAttributes>name);
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Inject');
  }

  public propertyDecorator(target: Object, propertyKey: PropertyKeyType): void {
    console.log('Inject');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('Inject');
  }

  public parameterDecorator(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('Inject');
  }

}

export const Inject = Decorator.create(InjectAnnotation);
