import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

export interface ValueAnnotationAttributes {
  value: string;
}

export class ValueAnnotation extends Annotation {
  public readonly attributes: ValueAnnotationAttributes = {
    value: undefined,
  };

  public constructor(value: string | ValueAnnotationAttributes) {
    super();
    if (undefined === value) {
      throw new Error(`value attribute must be specified.`);
    }

    if (TypeUtil.isString(value)) {
      this.attributes.value = <string>value;
    } else {
      this.copyAttributes(this.attributes, <ValueAnnotationAttributes>value);
    }
  }

  public propertyDecorator(target: Object, propertyKey: PropertyKeyType): void {
    console.log('Value');
  }

  public parameterDecorator(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('Value');
  }
}

export const Value = Decorator.create(ValueAnnotation);
